@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.7 2006/02/05 01:59:27 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.16 2006/06/26 08:25:17 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -81,7 +81,8 @@ void do_child(int unix_fd, int inet_fd)
 	struct timeval timeout;
 	static int connected;
 	int connections_count = 0;	/* used if child_max_connections > 0 */
-
+	int first_ready_for_query_received;		/* for master/slave mode */
+	int found;
 
 	pool_debug("I am %d", getpid());
 
@@ -120,6 +121,8 @@ void do_child(int unix_fd, int inet_fd)
 	timeout.tv_sec = pool_config.child_life_time;
 	timeout.tv_usec = 0;
 
+	init_prepared_list();
+
 	for (;;)
 	{
 		int connection_reuse = 1;
@@ -217,19 +220,67 @@ void do_child(int unix_fd, int inet_fd)
 		 * we need to connect to the backend and send the startup packet.
 		 */
 
+		first_ready_for_query_received = 0;		/* for master/slave mode */
+
 		/* look for existing connection */
-		if ((backend = pool_get_cp(sp->user, sp->database, sp->major)) == NULL)
+		found = 0;
+		backend = pool_get_cp(sp->user, sp->database, sp->major);
+
+		if (backend != NULL)
+		{
+			found = 1;
+
+			/* existing connection associated with same user/database/major found.
+			 * however we should make sure that the startup packet contents identical.
+			 * OPTION data and others might be different.
+			 */
+			if (sp->len != backend->slots[0]->sp->len)
+			{
+				pool_debug("pool_process_query: connection exists but startup packet length is not identical");
+				found = 0;
+			}
+			else if(memcmp(sp->startup_packet, backend->slots[0]->sp->startup_packet, sp->len) != 0)
+			{
+				pool_debug("pool_process_query: connection exists but startup packet contents is not identical");
+				found = 0;
+			}
+
+			if (found == 0)
+			{
+				/* we need to discard existing connection since startup packet is different */
+				pool_discard_cp(sp->user, sp->database, sp->major);
+				backend = NULL;
+			}
+		}
+
+		if (backend == NULL)
 		{
 			/* create a new connection to backend */
 			connection_reuse = 0;
 
 			if ((backend = connect_backend(sp, frontend)) == NULL)
 				continue;
+
+			/* in master/slave mode, the first "ready for query"
+			 * packet should be treated as if we were not in the
+			 * mode
+			 */
+			if (MASTER_SLAVE)
+				first_ready_for_query_received = 1;
 		}
+
 		else
 		{
-			/* we don't need Startup Packet info anymore */
-			pool_free_startup_packet(sp);
+			/*
+			 * save startup packet info
+			 */
+			pool_free_startup_packet(backend->slots[0]->sp);
+			backend->slots[0]->sp = sp;
+
+			if (DUAL_MODE)
+			{
+				backend->slots[1]->sp = sp;
+			}
 
 			/* reuse existing connection to backend */
 
@@ -281,7 +332,7 @@ void do_child(int unix_fd, int inet_fd)
 		{
 			POOL_STATUS status;
 
-			status = pool_process_query(frontend, backend, 0);
+			status = pool_process_query(frontend, backend, 0, first_ready_for_query_received);
 
 			sp = MASTER_CONNECTION(backend)->sp;
 
@@ -308,7 +359,7 @@ void do_child(int unix_fd, int inet_fd)
 						POOL_STATUS status1;
 
 						/* send reset request to backend */
-						status1 = pool_process_query(frontend, backend, 1);
+						status1 = pool_process_query(frontend, backend, 1, 0);
 						pool_close(frontend);
 
 						/* if we detect errors on resetting connection, we need to discard
@@ -845,6 +896,9 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	return backend;
 }
 
+/*
+ * signal handler for SIGINT and SIGQUUT
+ */
 static RETSIGTYPE die(int sig)
 {
 	exit_request = 1;
@@ -874,6 +928,10 @@ static RETSIGTYPE die(int sig)
 	exit(0);
 }
 
+/*
+ * signal handler for SIGHUP
+ * close all idle connections
+ */
 static RETSIGTYPE close_idle_connection(int sig)
 {
 	int i;
@@ -905,11 +963,23 @@ static RETSIGTYPE close_idle_connection(int sig)
 	}
 }
 
+/*
+ * send frontend exiting messages to all connections.
+ * this is called when child life time expires or child max connections expires.
+ */
 static void send_frontend_exits(void)
 {
 	int i;
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	for (i=0;i<pool_config.max_pool;i++, p++)
 	{
 		if (!MASTER_CONNECTION(p))
@@ -918,6 +988,8 @@ static void send_frontend_exits(void)
 			continue;
 		pool_send_frontend_exits(p);
 	}
+
+	POOL_SETMASK(&oldmask);
 }
 
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
@@ -979,7 +1051,7 @@ int health_check(void)
 	mysp.len = htonl(296);
 	mysp.sp.protoVersion = htonl(PROTO_MAJOR_V2 << 16);
 	strcpy(mysp.sp.database, "template1");
-	strcpy(mysp.sp.user, pool_config.health_check_user);
+ 	strncpy(mysp.sp.user, pool_config.health_check_user, sizeof(mysp.sp.user) - 1);
 	*mysp.sp.options = '\0';
 	*mysp.sp.unused = '\0';
 	*mysp.sp.tty = '\0';
