@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.3 2006/02/05 01:59:27 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.5 2006/06/26 08:25:17 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -66,6 +66,12 @@ int pool_init_cp(void)
 */
 POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor)
 {
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
 	int i;
 
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
@@ -76,6 +82,8 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor)
 		return NULL;
 	}
 
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	for (i=0;i<pool_config.max_pool;i++)
 	{
 		if (MASTER_CONNECTION(p) &&
@@ -86,10 +94,13 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor)
 		{
 			/* mark this connection is under use */
 			MASTER_CONNECTION(p)->closetime = 0;
+			POOL_SETMASK(&oldmask);
 			return p;
 		}
 		p++;
 	}
+
+	POOL_SETMASK(&oldmask);
 	return NULL;
 }
 
@@ -441,7 +452,7 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 		create_cp(SECONDARY_CONNECTION(p), 1);
 
 		/* initialize Paramter Status save structure */
-		if (pool_init_params(&MASTER(p)->params))
+		if (pool_init_params(&SECONDARY(p)->params))
 		{
 			return NULL;
 		}
