@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_error.c,v 1.3 2006/02/05 01:59:27 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_error.c,v 1.5 2006/04/28 07:37:39 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -43,6 +43,14 @@ void pool_error(const char *fmt,...)
 	char		*fmt2;
 #endif
 
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	if (pool_config.print_timestamp)
 #ifdef HAVE_ASPRINTF
 	  asprintf(&fmt2, "%s ERROR: pid %d: %s\n", nowsec(), (int)getpid(), fmt);
@@ -67,6 +75,8 @@ void pool_error(const char *fmt,...)
 	va_end(ap);
 	fprintf(stderr, "\n");
 #endif
+
+	POOL_SETMASK(&oldmask);
 }
 
 void pool_debug(const char *fmt,...)
@@ -76,9 +86,17 @@ void pool_debug(const char *fmt,...)
 	char		*fmt2;
 #endif
 
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
 	if (!debug)
 		return;
 
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	if (pool_config.print_timestamp)
 #ifdef HAVE_ASPRINTF
 	  asprintf(&fmt2, "%s DEBUG: pid %d: %s\n", nowsec(), (int)getpid(), fmt);
@@ -103,6 +121,8 @@ void pool_debug(const char *fmt,...)
 	va_end(ap);
 	fprintf(stderr, "\n");
 #endif
+
+	POOL_SETMASK(&oldmask);
 }
 
 void pool_log(const char *fmt,...)
@@ -112,6 +132,14 @@ void pool_log(const char *fmt,...)
 	char		*fmt2;
 #endif
 
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	if (pool_config.print_timestamp)
 #ifdef HAVE_ASPRINTF
 	  asprintf(&fmt2, "%s LOG:   pid %d: %s\n", nowsec(), (int)getpid(), fmt);
@@ -136,6 +164,8 @@ void pool_log(const char *fmt,...)
 	va_end(ap);
 	fprintf(stderr, "\n");
 #endif
+
+	POOL_SETMASK(&oldmask);
 }
 
 static char *nowsec(void)
