@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.24 2006/05/10 10:15:17 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.29 2006/07/21 04:25:42 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -571,7 +571,15 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		string = query;
 	}
 
-	pool_debug("Query: %s", string);
+	/* log query to log file if neccessary */
+	if (pool_config.log_statement)
+	{
+		pool_log("statement: %s", string);
+	}
+	else
+	{
+		pool_debug("statement: %s", string);
+	}
 
 	/*
 	 * if this is DROP DATABASE command, send HUP signal to parent and
@@ -739,7 +747,6 @@ static POOL_STATUS Execute(POOL_CONNECTION *frontend,
 	int len;
 	int sendlen;
 	int i;
-	char kind0 = 0;
 	char kind;
 	int status;
 
@@ -773,38 +780,35 @@ static POOL_STATUS Execute(POOL_CONNECTION *frontend,
 			return POOL_ERROR;
 		}
 
-		status = pool_read(cp, &kind, sizeof(kind));
-		if (status < 0)
+		if (!REPLICATION)
+			break;
+		else if (pool_config.replication_strict)
 		{
-			pool_debug("Execute: pool_read returns error");
+			pool_debug("waiting for backend completing the query");
+			if (synchronize(cp))
+				return POOL_END;
+		}
+	}
+
+	while ((kind = pool_read_kind(backend)),
+		   (kind != 'C' && kind != 'E' && kind != 'l' && kind != 's'))
+	{
+		if (kind < 0)
+		{
+			pool_error("Execute: pool_read_kind error");
 			return POOL_ERROR;
 		}
 
-		if (i == 0)
-			kind0 = kind;
-		else
-		{
-			if (kind != kind0)
-			{
-				pool_error("Execute: kind does not match kind0:%c kind:%c", kind0, kind);
-				return POOL_ERROR;
-			}
-		}
-
-		if (!REPLICATION)
-			break;
+		status = SimpleForwardToFrontend(kind, frontend, backend);
+		if (status != POOL_CONTINUE)
+			return status;
+		pool_flush(frontend);
 	}
-
-	if (kind == 0)
-	{
-		pool_error("Execute: kind is 0!");
-		return POOL_ERROR;
-	}
-
-	SimpleForwardToFrontend(kind, frontend, backend);
-	status = pool_flush(frontend);
+	status = SimpleForwardToFrontend(kind, frontend, backend);
 	if (status != POOL_CONTINUE)
-		return POOL_ERROR;
+		return status;
+	pool_flush(frontend);
+
 	return POOL_CONTINUE;
 }
 
@@ -2364,12 +2368,42 @@ POOL_STATUS SimpleForwardToFrontend(char kind, POOL_CONNECTION *frontend, POOL_C
 	if (pool_write(frontend, p, len))
 		return POOL_END;
 
-	if (kind == 'E')		/* error response? */
+	if (kind == 'A')	/* notification response */
+	{
+		pool_flush(frontend);	/* we need to immediately notice to frontend */
+	}
+	else if (kind == 'E')		/* error response? */
 	{
 		int i, k;
 		int res1, res2;
 		char *p1;
 
+		/*
+		 * check if the error was PANIC or FATAL. If so, we just flush
+		 * the message and exit since the backend will close the
+		 * channel immediately.
+		 */
+		for (;;)
+		{
+			char e;
+
+			e = *p++;
+			if (e == '\0')
+				break;
+
+			if (e == 'S' && (strcasecmp("PANIC", p) == 0 || strcasecmp("FATAL", p) == 0))
+			{
+				pool_flush(frontend);
+				return POOL_END;
+			}
+			else
+			{
+				while (*p++)
+					;
+				continue;
+			}
+		}
+
 		for (i = 0;i < backend->num;i++)
 		{
 			POOL_CONNECTION *cp = backend->slots[i]->con;
