@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool/pool_signal.h,v 1.1.1.1 2005/02/14 14:52:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_signal.h,v 1.3 2006/04/28 07:37:39 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Portions Copyright (c) 2003-2005, Tatsuo Ishii
+ * Portions Copyright (c) 2003-2006,	PgPool Global Development Group
  * Portions Copyright (c) 2004, PostgreSQL Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
@@ -38,6 +38,7 @@ extern sigset_t UnBlockSig,
 			AuthBlockSig;
 
 #define POOL_SETMASK(mask)	sigprocmask(SIG_SETMASK, mask, NULL)
+#define POOL_SETMASK2(mask, oldmask)	sigprocmask(SIG_SETMASK, mask, oldmask)
 #else
 extern int	UnBlockSig,
 			BlockSig,
@@ -45,8 +46,10 @@ extern int	UnBlockSig,
 
 #ifndef WIN32
 #define POOL_SETMASK(mask)	sigsetmask(*((int*)(mask)))
+#define POOL_SETMASK2(mask, oldmask)	do {oldmask = POOL_SETMASK(mask)} while (0)
 #else
 #define POOL_SETMASK(mask)		pqsigsetmask(*((int*)(mask)))
+#define POOL_SETMASK2(mask, oldmask)	do {oldmask = POOL_SETMASK(mask)} while (0)
 int			pqsigsetmask(int mask);
 #endif
 #endif
